﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;

namespace SampleApp
{
    class Function
    {
        public Record RecordInfo;

        /// <summary>
        /// 定数
        /// </summary>
        public static class RA3100Const
        {
            public const int MaxSlot = 9;
            public const string XMLFileName = "RecordingInfo.xml";
            public const string PrinterFolderName = "Printer";
            public const string SSDFolderName = "Storage";
            public const string MemoryFolderName = "Memory";
            public const int StatusBitTriggr = 0;
            public const int StatusBitMark = 1;
        }

        /// <summary>
        /// 記録データ
        /// </summary>
        public class RecordData
        {
            public List<List<double>> Value = new List<List<double>>();
            public List<string> HeaderText = new List<string>();
            public List<string> ReadFileName = new List<string>();
            public long FileSampleCount = 0;
            public ulong MemoryTriggerPoint = 0;
        }

        /// <summary>
        /// 記録データ解析用設定
        /// </summary>
        private class RecordDataParameters
        {
            public string FolderPath = string.Empty;
            public string FileName = string.Empty;
            public int FileNumber = 0;
            public long ReadSamplePoint = 0;
            public long ReadSampleCount = 0;
            public bool PhysicalActive = false;
            public BinaryFormat BinaryFormat = new BinaryFormat();
        }

        /// <summary>
        /// バイナリデータフォーマット
        /// </summary>
        private class BinaryFormat
        {
            public int OneSampleSize = 0;
            public RecordDataType RecordDataType = RecordDataType.Normal;
            public bool StatusDataActive = false;
        }

        /// <summary>
        /// XMLファイルから記録情報を取得する
        /// </summary>
        /// <param name="folderPath">記録データフォルダパス</param>
        /// <returns>実行結果</returns>
        public bool GetRecordInfo(string folderPath)
        {
            bool ret = true;
            string filePath = folderPath + "\\" + RA3100Const.XMLFileName;

            try
            {
                //ファイルオープン
                using (FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read))
                {
                    // シリアライザーのインスタンス作成
                    XmlSerializer serializer = new XmlSerializer(typeof(Record));
                    //逆シリアライズしてクラスに値を格納
                    RecordInfo = (Record)serializer.Deserialize(fs);
                }
            }
            catch (Exception)
            {
                ret = false;
                RecordInfo = null;
            }

            return ret;
        }

        /// <summary>
        /// 1サンプルのバイナリデータを記録データに変換
        /// </summary>
        /// <param name="buf">1サンプルのバイナリデータBuffer</param>
        /// <param name="binaryFormat">バイナリデータフォーマット設定</param>
        /// <param name="physicalActive">物理変換設定</param>
        /// <param name="oneSampleData">記録データ変換結果(データ値)</param>
        /// <param name="oneSampleText">記録データ変換結果(ヘッダーテキスト)</param>
        /// <returns>実行結果</returns>
        private bool ConvertOneSampleInfo(byte[] buf, BinaryFormat binaryFormat, bool physicalActive, ref List<double> oneSampleData, ref List<string> oneSampleText)
        {
            bool ret = true;

            try
            {
                //スロット情報取得
                foreach (Slot slot in RecordInfo.CHInfo.Slots)
                {
                    //計測用モジュールの確認
                    if (slot.GetType() == typeof(MeasurementModule))
                    {
                        MeasurementModule module = slot as MeasurementModule;
                        //CH情報取得
                        foreach (Channel ch in module.Channels)
                        {
                            //バイナリデータ有無の確認
                            if (ch.BinaryData != null)
                            {
                                //アナログモジュール
                                if (ch.BinaryData.Offset.GetType() == typeof(AnalogModule))
                                {
                                    AnalogModule analogOffset = ch.BinaryData.Offset as AnalogModule;

                                    //Normal or P-P
                                    if (binaryFormat.RecordDataType == RecordDataType.Normal)
                                    {
                                        //Normal
                                        BinaryData binaryData = new BinaryData();   //バイナリデータタイプ
                                        long value = 0;                             //bufから読み込んだ値
                                        double recordData = 0;                      //記録データ値
                                        string unit = string.Empty;                 //単位

                                        //バイナリデータタイプの確認
                                        switch (ch.BinaryData.GetType().Name)
                                        {
                                            case nameof(Int16):
                                                //バイナリデータタイプをInt16に指定
                                                binaryData = ch.BinaryData as Int16;
                                                //bufから値を取得
                                                value = (long)BitConverter.ToInt16(buf, analogOffset.Normal);
                                                break;

                                            case nameof(UInt16):
                                                //バイナリデータタイプをUInt16に指定
                                                binaryData = ch.BinaryData as UInt16;
                                                //bufから値を取得
                                                value = (long)BitConverter.ToUInt16(buf, analogOffset.Normal);
                                                break;
                                                
                                            case nameof(Int32):
                                                //バイナリデータタイプをInt32に指定
                                                binaryData = ch.BinaryData as Int32;
                                                //bufから値を取得
                                                value = (long)BitConverter.ToInt32(buf, analogOffset.Normal);
                                                break;

                                            case nameof(UInt32):
                                                //バイナリデータタイプをUInt32に指定
                                                binaryData = ch.BinaryData as UInt32;
                                                //bufから値を取得
                                                value = (long)BitConverter.ToUInt32(buf, analogOffset.Normal);
                                                break;

                                            default:
                                                return false;
                                        }

                                        //変換
                                        if (physicalActive)
                                        {
                                            //物理量変換あり
                                            recordData = ((double)value * binaryData.ToPhysical.Gain) + binaryData.ToPhysical.Offset;
                                            unit = "[" + binaryData.ToPhysical.Unit + "]";
                                        }
                                        else
                                        {
                                            //物理量変換なし
                                            recordData = ((double)value * binaryData.ToElectricity.Gain) + binaryData.ToElectricity.Offset;
                                            unit = "[" + binaryData.ToElectricity.Unit + "]";
                                        }

                                        //記録データ値
                                        oneSampleData.Add(recordData);

                                        //ヘッダーテキスト
                                        if (ch.Name != "")
                                        {
                                            oneSampleText.Add(ch.Name + unit);
                                        }
                                        else
                                        {
                                            oneSampleText.Add("S" + slot.No.ToString() + "-CH" + ch.No + unit);
                                        }
                                    }
                                    else
                                    {
                                        //P-P
                                        BinaryData binaryData = new BinaryData();   //バイナリデータタイプ
                                        long minValue = 0;                          //bufから読み込んだ値(Min)
                                        long maxValue = 0;                          //bufから読み込んだ値(Max)
                                        double minRecordData = 0;                   //記録データ値(Min)
                                        double maxRecordData = 0;                   //記録データ値(Max)
                                        string unit = string.Empty;                 //単位

                                        //バイナリデータタイプの確認
                                        switch (ch.BinaryData.GetType().Name)
                                        {
                                            case nameof(Int16):
                                                //バイナリデータタイプをInt16に指定
                                                binaryData = ch.BinaryData as Int16;
                                                //bufから値を取得
                                                minValue = (long)BitConverter.ToInt16(buf, analogOffset.PP1);
                                                maxValue = (long)BitConverter.ToInt16(buf, analogOffset.PP2);
                                                break;

                                            case nameof(UInt16):
                                                //バイナリデータタイプをUInt16に指定
                                                binaryData = ch.BinaryData as UInt16;
                                                //bufから値を取得
                                                minValue = (long)BitConverter.ToUInt16(buf, analogOffset.PP1);
                                                maxValue = (long)BitConverter.ToUInt16(buf, analogOffset.PP2);
                                                break;

                                            case nameof(Int32):
                                                //バイナリデータタイプをInt32に指定
                                                binaryData = ch.BinaryData as Int32;
                                                //bufから値を取得
                                                minValue = (long)BitConverter.ToInt32(buf, analogOffset.PP1);
                                                maxValue = (long)BitConverter.ToInt32(buf, analogOffset.PP2);
                                                break;

                                            case nameof(UInt32):
                                                //バイナリデータタイプをUInt32に指定
                                                binaryData = ch.BinaryData as UInt32;
                                                //bufから値を取得
                                                minValue = (long)BitConverter.ToUInt32(buf, analogOffset.PP1);
                                                maxValue = (long)BitConverter.ToUInt32(buf, analogOffset.PP2);
                                                break;

                                            default:
                                                return false;
                                        }

                                        //変換
                                        if (physicalActive)
                                        {
                                            //物理量変換あり
                                            minRecordData = ((double)minValue * binaryData.ToPhysical.Gain) + binaryData.ToPhysical.Offset;
                                            maxRecordData = ((double)maxValue * binaryData.ToPhysical.Gain) + binaryData.ToPhysical.Offset;
                                            unit = "[" + binaryData.ToPhysical.Unit + "]";
                                        }
                                        else
                                        {
                                            //物理量変換なし
                                            minRecordData = ((double)minValue * binaryData.ToElectricity.Gain) + binaryData.ToElectricity.Offset;
                                            maxRecordData = ((double)maxValue * binaryData.ToElectricity.Gain) + binaryData.ToElectricity.Offset;
                                            unit = "[" + binaryData.ToElectricity.Unit + "]";
                                        }

                                        //記録データ値(Min)
                                        oneSampleData.Add(minRecordData);

                                        //記録データ値(MAx)
                                        oneSampleData.Add(maxRecordData);

                                        //ヘッダーテキスト
                                        if (ch.Name != "")
                                        {
                                            oneSampleText.Add(ch.Name + " Min" + unit);
                                            oneSampleText.Add(ch.Name + " Max" + unit);
                                        }
                                        else
                                        {
                                            oneSampleText.Add("S" + slot.No.ToString() + "-CH" + ch.No + " Min" + unit);
                                            oneSampleText.Add("S" + slot.No.ToString() + "-CH" + ch.No + " Max" + unit);
                                        }
                                    }
                                }

                                //ロジックモジュール
                                if (ch.BinaryData.Offset.GetType() == typeof(LogicModule))
                                {
                                    LogicModule logicOffset = ch.BinaryData.Offset as LogicModule;

                                    //Normal or P-P
                                    if (binaryFormat.RecordDataType == RecordDataType.Normal)
                                    {
                                        //Normal
                                        BinaryData binaryData = new BinaryData();   //バイナリデータタイプ
                                        long value = 0;                             //bufから読み込んだ値(全8CHのデータ)

                                        //バイナリデータタイプの確認
                                        switch (ch.BinaryData.GetType().Name)
                                        {
                                            case nameof(BitFlag):
                                                //バイナリデータタイプをBitFlagに指定
                                                binaryData = ch.BinaryData as BitFlag;
                                                //bufから値を取得
                                                value = (long)BitConverter.ToChar(buf, logicOffset.Normal);
                                                break;

                                            default:
                                                return false;
                                        }

                                        //CH毎に値を取得
                                        for (int i = 0; i < 8; i++)
                                        {
                                            //CH1～8
                                            oneSampleData.Add((value & (1 << i)) >> i);

                                            //ヘッダーテキスト
                                            if (ch.Name != "")
                                            {
                                                oneSampleText.Add(ch.Name + (i + 1).ToString());
                                            }
                                            else
                                            {
                                                oneSampleText.Add("S" + slot.No.ToString() + "-CH" + ch.No + (i + 1).ToString());
                                            }
                                        }
                                    }
                                    else
                                    {
                                        //P-P
                                        BinaryData binaryData = new BinaryData();   //バイナリデータタイプ
                                        long value = 0;                             //bufから読み込んだ値(全CHのデータ)
                                        long flagValue = 0;                         //bufから読み込んだ値(全CHの変化フラグデータ)

                                        //バイナリデータタイプの確認
                                        switch (ch.BinaryData.GetType().Name)
                                        {
                                            case nameof(BitFlag):
                                                //バイナリデータタイプをBitFlagに指定
                                                binaryData = ch.BinaryData as BitFlag;
                                                //bufから値を取得
                                                value = (long)BitConverter.ToChar(buf, logicOffset.PP1);
                                                flagValue = (long)BitConverter.ToChar(buf, logicOffset.PP2);
                                                break;

                                            default:
                                                return false;
                                        }

                                        //CH毎に値を取得
                                        for (int i = 0; i < 8; i++)
                                        {
                                            //CH1～8
                                            oneSampleData.Add((value & (1 << i)) >> i);

                                            //CH1～8変化フラグ
                                            oneSampleData.Add((flagValue & (1 << i)) >> i);

                                            //ヘッダーテキスト
                                            if (ch.Name != "")
                                            {
                                                oneSampleText.Add(ch.Name + (i + 1).ToString());
                                                oneSampleText.Add(ch.Name + (i + 1).ToString() + " Flag");
                                            }
                                            else
                                            {
                                                oneSampleText.Add("S" + slot.No.ToString() + "-CH" + ch.No + (i + 1).ToString());
                                                oneSampleText.Add("S" + slot.No.ToString() + "-CH" + ch.No + (i + 1).ToString() + " Flag");
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //ステータス情報取得
                if (binaryFormat.StatusDataActive)
                {
                    //バイナリデータ有無の確認
                    if (RecordInfo.StatusInfo.BinaryData != null)
                    {
                        short Status;   //Status値

                        //Normal or P-P
                        if (binaryFormat.RecordDataType == RecordDataType.Normal)
                        {
                            //Normal
                            //bufから値を取得
                            Status = BitConverter.ToInt16(buf, RecordInfo.StatusInfo.BinaryData.Offset.Normal);
                        }
                        else
                        {
                            //P-P
                            //bufから値を取得
                            Status = BitConverter.ToInt16(buf, RecordInfo.StatusInfo.BinaryData.Offset.PP1);
                        }

                        //Triggr情報(Statusの0bit)
                        oneSampleData.Add((Status & (1 << RA3100Const.StatusBitTriggr)) >> RA3100Const.StatusBitTriggr);
                        oneSampleText.Add("Triggr");

                        //Mark情報(Statusの1bit)
                        oneSampleData.Add((Status & (1 << RA3100Const.StatusBitMark)) >> RA3100Const.StatusBitMark);
                        oneSampleText.Add("Mark");

                        //システム予約(Statusのその他bit)

                    }
                    else
                    {
                        return false;
                    }
                }
            }
            catch (Exception)
            {
                return false;
            }

            return ret;
        }

        /// <summary>
        /// BINファイルから記録データを取得する
        /// </summary>
        /// <param name="filePath">BINファイルパス</param>
        /// <param name="sampleOffset">オフセットサンプル数</param>
        /// <param name="readSampleCount">読み込むサンプル数</param>
        /// <param name="physicalActive">物理変換設定</param>
        /// <param name="binaryFormat">バイナリデータフォーマット設定</param>
        /// <param name="recordData">記録データ</param>
        /// <returns>実行結果</returns>
        private bool ReadBinaryFile(string filePath, long sampleOffset, long readSampleCount, bool physicalActive, BinaryFormat binaryFormat, ref RecordData recordData)
        {
            bool ret = true;

            long offsetSize = (long)sampleOffset * (long)binaryFormat.OneSampleSize;

            try
            {
                //ファイルオープン
                using (FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read))
                {
                    //読み込み位置へ移動
                    fs.Seek(offsetSize, SeekOrigin.Begin);

                    //指定サンプル数読み込み
                    for (long i = 0; i < readSampleCount; i++)
                    {
                        List<double> oneSampleData = new List<double>();
                        List<string> oneSampleText = new List<string>();
                        byte[] buf = new byte[binaryFormat.OneSampleSize + 1]; // データ格納用配列
                        int readSize;

                        //1サンプル読み込み
                        readSize = fs.Read(buf, 0, binaryFormat.OneSampleSize);
                        if (binaryFormat.OneSampleSize > readSize)
                        {
                            break;
                        }

                        //次回の読み込み位置計算
                        offsetSize += (long)readSize;

                        //読み込んだバイナリデータを記録データに変換
                        if (!ConvertOneSampleInfo(buf, binaryFormat, physicalActive, ref oneSampleData, ref oneSampleText))
                        {
                            break;
                        }

                        //記録データ
                        recordData.Value.Add(oneSampleData);

                        //記録ヘッダーテキスト
                        if (recordData.HeaderText.Count == 0)
                        {
                            foreach (string headerText in oneSampleText)
                            {
                                recordData.HeaderText.Add(headerText);
                            }
                        }
                    }
                }
            }
            catch (Exception)
            {
                return false;
            }

            return ret;
        }

        /// <summary>
        /// BINファイルから記録データを取得する
        /// </summary>
        /// <param name="recDataParam">記録データ取得用設定</param>
        /// <param name="recordData">記録データ</param>
        /// <returns>実行結果</returns>
        private bool GetRecordData(RecordDataParameters recDataParam, ref RecordData recordData)
        {
            bool ret = true;
            string filePath;
            int fileNumber = 0;
            long sampleOffset = recDataParam.ReadSamplePoint - 1;
            long readSampleCount = recDataParam.ReadSampleCount;

            //記録データ初期化
            recordData = new RecordData();

            //読み込むBINファイルを確認
            if (recDataParam.FileNumber == 0)
            {
                //ファイル番号が指定されていない(0指定)の場合は、1.binから順に読み込む

                //BINファイルから指定サンプル数を読み込む
                while (readSampleCount > 0)
                {
                    //ファイル名インクリメント
                    fileNumber++;
                    filePath = recDataParam.FolderPath + "\\" + recDataParam.FileName + fileNumber + ".bin";

                    //ファイルの存在確認
                    if (File.Exists(filePath))
                    {
                        FileInfo file = new FileInfo(filePath);
                        long fileSampleCount = (long)file.Length / (long)recDataParam.BinaryFormat.OneSampleSize;

                        //読み込むBINファイルを特定する
                        if (fileSampleCount > sampleOffset)
                        {
                            //ファイル名
                            string[] del = { "\\" };
                            string[] fileName = filePath.Split(del, StringSplitOptions.None);
                            recordData.ReadFileName.Add(fileName.Last());

                            //1ファイルのサンプル数
                            if (recordData.FileSampleCount == 0)
                            {
                                recordData.FileSampleCount = fileSampleCount;
                            }

                            //BINファイルから記録データを取得
                            ReadBinaryFile(filePath, sampleOffset, readSampleCount, recDataParam.PhysicalActive, recDataParam.BinaryFormat, ref recordData);

                            //オフセットクリア
                            sampleOffset = 0;

                            //残りの読み込むサンプル数を計算
                            readSampleCount -= (long)recordData.Value.Count;
                        }
                        else
                        {
                            //1ファイル分オフセットを減らす
                            sampleOffset -= fileSampleCount;
                        }
                    }
                    else
                    {
                        break;
                    }
                }
            }
            else
            {
                //指定BINファイルから記録データを取得

                //ファイルパス
                filePath = recDataParam.FolderPath + "\\" + recDataParam.FileName + recDataParam.FileNumber + ".bin";

                //ファイルの存在確認
                if (File.Exists(filePath))
                {
                    //ファイル名
                    string[] del = { "\\" };
                    string[] fileName = filePath.Split(del, StringSplitOptions.None);
                    recordData.ReadFileName.Add(fileName.Last());

                    //1ファイルのサンプル数
                    if (recordData.FileSampleCount == 0)
                    {
                        FileInfo file = new FileInfo(filePath);
                        long fileSampleCount = (long)file.Length / (long)recDataParam.BinaryFormat.OneSampleSize;
                        recordData.FileSampleCount = fileSampleCount;
                    }

                    //BINファイルから記録データを取得
                    ReadBinaryFile(filePath, sampleOffset, readSampleCount, recDataParam.PhysicalActive, recDataParam.BinaryFormat, ref recordData);
                }
                else
                {
                    return false;
                }
            }

            //記録データにサンプル番号を追加
            recordData.HeaderText.Insert(0, "No.");
            for (int i = 0; i < recordData.Value.Count; i++)
            {
                recordData.Value[i].Insert(0, (long)i + recDataParam.ReadSamplePoint);
            }

            return ret;
        }

        /// <summary>
        /// プリンタ記録のBINファイルから記録データを取得する
        /// </summary>
        /// <param name="folderPath">記録データフォルダパス</param>
        /// <param name="readSamplePoint">読み込む開始サンプル点</param>
        /// <param name="readSampleCount">読み込むサンプル数<</param>
        /// <param name="physicalActive">物理変換設定</param>
        /// <param name="recordData">記録データ</param>
        /// <returns>実行結果</returns>
        public bool GetPrinterRecordData(string folderPath, long readSamplePoint, long readSampleCount, bool physicalActive, ref RecordData recordData)
        {
            bool ret = true;

            if ((RecordInfo != null) && (RecordInfo.PrinterInfo != null))
            {
                RecordDataParameters recDataParam = new RecordDataParameters();

                //範囲確認
                if ((readSamplePoint < 1) || (readSampleCount < 1))
                {
                    return false;
                }

                //ファイルパスを指定
                recDataParam.FolderPath = folderPath + "\\" + RA3100Const.PrinterFolderName;
                recDataParam.FileName = "P";

                //読み込むサンプル位置、サンプル数を指定
                recDataParam.ReadSamplePoint = readSamplePoint;
                recDataParam.ReadSampleCount = readSampleCount;

                //変換方法を指定
                recDataParam.PhysicalActive = physicalActive;

                //バイナリデータのフォーマットを指定
                recDataParam.BinaryFormat.OneSampleSize = RecordInfo.PrinterInfo.OneSampleSize;
                recDataParam.BinaryFormat.RecordDataType = RecordInfo.PrinterInfo.DataFormat;
                recDataParam.BinaryFormat.StatusDataActive = true;     //プリンタ記録にはバイナリデータにステータス情報(トリガ、マーク)が付加されているため"true"

                //記録データの読み込み開始
                ret = GetRecordData(recDataParam, ref recordData);
            }
            else
            {
                ret = false;
            }

            return ret;
        }

        /// <summary>
        /// SSD記録のBINファイルから記録データを取得する
        /// </summary>
        /// <param name="folderPath">記録データフォルダパス</param>
        /// <param name="readSamplePoint">読み込む開始サンプル点</param>
        /// <param name="readSampleCount">読み込むサンプル数<</param>
        /// <param name="physicalActive">物理変換設定</param>
        /// <param name="recordData">記録データ</param>
        /// <returns>実行結果</returns>
        public bool GetSSDRecordData(string folderPath, long readSamplePoint, long readSampleCount, bool physicalActive, ref RecordData recordData)
        {
            bool ret = true;

            if ((RecordInfo != null) && (RecordInfo.SSDInfo != null))
            {
                RecordDataParameters recDataParam = new RecordDataParameters();

                //範囲確認
                if ((readSamplePoint < 1) || (readSampleCount < 1))
                {
                    return false;
                }

                //ファイルパスを指定
                recDataParam.FolderPath = folderPath + "\\" + RA3100Const.SSDFolderName;
                recDataParam.FileName = "S";

                //読み込むサンプル位置、サンプル数を指定
                recDataParam.ReadSamplePoint = readSamplePoint;
                recDataParam.ReadSampleCount = readSampleCount;

                //変換方法を指定
                recDataParam.PhysicalActive = physicalActive;

                //バイナリデータのフォーマットを指定
                recDataParam.BinaryFormat.OneSampleSize = RecordInfo.SSDInfo.OneSampleSize;
                recDataParam.BinaryFormat.RecordDataType = RecordInfo.SSDInfo.DataFormat;
                recDataParam.BinaryFormat.StatusDataActive = true;     //SSD記録にはバイナリデータにステータス情報(トリガ、マーク)が付加されているため"true"

                //記録データの読み込み開始
                ret = GetRecordData(recDataParam, ref recordData);
            }
            else
            {
                ret = false;
            }

            return ret;
        }

        /// <summary>
        /// メモリ記録のBINファイルから記録データを取得する
        /// </summary>
        /// <param name="folderPath">記録データフォルダパス</param>
        /// <param name="readSamplePoint">読み込む開始サンプル点</param>
        /// <param name="readSampleCount">読み込むサンプル数<</param>
        /// <param name="physicalActive">物理変換設定</param>
        /// <param name="readBlockNumber">読み込むブロックNo.</param>
        /// <param name="recordData">記録データ</param>
        /// <returns>実行結果</returns>
        public bool GetMemoryRecordData(string folderPath, long readSamplePoint, long readSampleCount, bool physicalActive, int readBlockNumber, ref RecordData recordData)
        {
            bool ret = true;

            if ((RecordInfo != null) && (RecordInfo.MemoryInfo != null))
            {
                RecordDataParameters recDataParam = new RecordDataParameters();

                //範囲確認
                if ((readSamplePoint < 1) || (readSampleCount < 1) || (readBlockNumber < 1))
                {
                    return false;
                }

                //ファイルパスを指定
                recDataParam.FolderPath = folderPath + "\\" + RA3100Const.MemoryFolderName;
                recDataParam.FileName = "M";

                //読み込むブロックが存在するか確認
                if (RecordInfo.MemoryInfo.RecordedBlocks != null)
                {
                    if (RecordInfo.MemoryInfo.RecordedBlocks.Length < readBlockNumber)
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }

                //読み込むブロック番号計算
                int blockNumber = RecordInfo.MemoryInfo.FirstBlockNo;
                for (int i = 0; i < readBlockNumber - 1; i++)
                {
                    blockNumber++;
                    if (blockNumber > RecordInfo.MemoryInfo.RecordedBlocks.Length)
                    {
                        blockNumber = 1;
                    }
                }

                //読み込むファイル番号(ブロック番号)を指定
                //メモリ記録ではブロック番号＝ファイル番号になる
                recDataParam.FileNumber = blockNumber;

                //読み込むサンプル位置、サンプル数を指定
                recDataParam.ReadSamplePoint = readSamplePoint;
                recDataParam.ReadSampleCount = readSampleCount;

                //変換方法を指定
                recDataParam.PhysicalActive = physicalActive;

                //バイナリデータのフォーマットを指定
                recDataParam.BinaryFormat.OneSampleSize = RecordInfo.MemoryInfo.OneSampleSize;
                recDataParam.BinaryFormat.RecordDataType = RecordInfo.MemoryInfo.DataFormat;
                recDataParam.BinaryFormat.StatusDataActive = false;     //メモリ記録にはバイナリデータにステータス情報(トリガ、マーク)が付加されていないため"false"

                //記録データの読み込み開始
                ret = GetRecordData(recDataParam, ref recordData);

                //メモリブロック毎にメモリトリガ位置情報が用意される
                recordData.MemoryTriggerPoint = RecordInfo.MemoryInfo.RecordedBlocks[blockNumber - 1].TriggerPoint;
            }
            else
            {
                ret = false;
            }

            return ret;
        }
    }
}
